/*
 * Copyright 2022-2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.jdbc;

import com.google.common.annotations.Beta;
import com.google.common.base.Preconditions;
import xyz.zhouxy.plusone.commons.collection.AbstractMapWrapper;
import xyz.zhouxy.plusone.commons.util.OptionalTools;
import xyz.zhouxy.plusone.commons.util.StringTools;

import java.util.*;

@Beta
public class DbRecord extends AbstractMapWrapper<String, Object, DbRecord> {

    public DbRecord() {
        super(new HashMap<>(), k -> Preconditions.checkArgument(StringTools.isNotBlank(k), "Key must has text."), null);
    }

    public DbRecord(Map<String, Object> map) {
        super(map, k -> Preconditions.checkArgument(StringTools.isNotBlank(k), "Key must has text."), null);
    }

    public Optional<String> getValueAsString(String key) {
        return this.getAndConvert(key);
    }

    public <T> List<T> getValueAsList(String key) {
        return this.<Collection<T>>getAndConvert(key)
                .map(l -> (l instanceof List) ? (List<T>) l : new ArrayList<>(l))
                .orElse(Collections.emptyList());
    }

    public <T> Set<T> getValueAsSet(String key) {
        return this.<Collection<T>>getAndConvert(key)
                .map(l -> (l instanceof Set) ? (Set<T>) l : new HashSet<>(l))
                .orElse(Collections.emptySet());
    }

    public OptionalInt getValueAsInt(String key) {
        return OptionalTools.toOptionalInt(this.getAndConvert(key));
    }

    public OptionalLong getValueAsLong(String key) {
        return OptionalTools.toOptionalLong(this.getAndConvert(key));
    }

    public OptionalDouble getValueAsDouble(String key) {
        return OptionalTools.toOptionalDouble(this.getAndConvert(key));
    }

    @Override
    protected DbRecord getSelf() {
        return this;
    }

    private static final String STR_PREFIX = DbRecord.class.getName() + '@';

    @Override
    public String toString() {
        return STR_PREFIX + super.toString();
    }
}
